import time
from calendar import monthrange

from netforce.model import Model, fields, get_model
from netforce.access import get_active_company

class ReportShop(Model):
    _name="clinic.report.shop"
    _string="Report Shop"
    _transient=True
    
    _fields={
        "date": fields.Date("Month", required=True),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'department_id': fields.Many2One("clinic.department","Departments"),
        'product_id': fields.Many2One("product","Product"),
    }

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get('date',time.strftime("%Y-%m-%d"))
        year,month=time.strftime("%Y-%m").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        date_from=defaults.get('date_from','%s-%s-01'%(year,month))
        date_to=defaults.get('date_to',"%s-%s-%s"%(year,month,total_day))
        product_id=defaults.get('product_id')
        branch_id=defaults.get('branch_id',None)
        if branch_id:
            branch_id=int(branch_id)
        department_id=defaults.get('department_id',None)
        if department_id:
            department_id=int(department_id)
        select_dpt=get_model('select.company').get_select()
        if select_dpt:
            if not branch_id:
                branch_id=select_dpt['branch_id']
            if not department_id:
                if select_dpt.get('department_ids'):
                    department_id=select_dpt['department_ids'][0]
                else:
                    department_id=select_dpt['department_id']
        res={
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'branch_id': branch_id,
            'department_id': department_id,
            'product_id': product_id,
        }
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults.get("date_from")
        date_to=defaults.get("date_to")
        date=defaults.get("date")
        branch_id=defaults.get("branch_id")
        department_id=defaults.get("department_id")
        product_id=defaults.get('product_id')
        if ids:
            obj=self.browse(ids)[0]
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            date=obj.date 
            date_from=obj.date_from
            date_to=obj.date_to
            product_id=obj.product_id.id
        ctx={
            'date_from': date_from,
            'date_to': date_to,
            'branch_id': branch_id,
            'department_id': department_id,
            'product_id': product_id,
        }
        lines=get_model('clinic.shop').get_shop_data(context=ctx)['records']
        company_name=company.name or ""
        if department_id:
            company_name+=' (%s)'%get_model('clinic.department').browse(department_id).name or ""
        elif branch_id:
            company_name+=' (%s)'%get_model('clinic.branch').browse(branch_id).name or ""
        total_fee=0
        total_srv=0
        total_epo=0
        total_lab=0
        total_dlz=0
        total_misc=0
        total_mdc=0
        slines=[]
        no=1
        for line in sorted(lines, key=lambda x: (x['date'])):
            total_fee+=line.get('fee',0)
            total_srv+=line.get('srv',0)
            total_epo+=line.get('epo',0)
            total_mdc+=line.get('mdc',0)
            total_lab+=line.get('lab',0)
            total_misc+=line.get('misc',0)
            total_dlz+=line.get('dlz_price',0)
            line['no']=no
            slines.append(line)
            no+=1
        data={
            'company_name': company_name,
            'branch_id': branch_id,
            'department_id': department_id,
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'lines': slines,
            'total_fee': total_fee,
            'total_srv': total_srv,
            'total_epo': total_epo,
            'total_mdc': total_mdc,
            'total_lab': total_lab,
            'total_misc': total_misc,
            'total_dlz': total_dlz,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data
    
    def onchange_datefrom(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data

ReportShop.register()
